@@ -10,7 +10,7 @@
 //! another.  Almost all platforms other than Microsoft Windows follow the
 //! [Itanium C++ ABI][itanium]'s rules for this.
 //!
-//! [itanium]: http://mentorembedded.github.io/cxx-abi/abi.html#mangling
+//! [itanium]: http://itanium-cxx-abi.github.io/cxx-abi/abi.html#mangle
 //!
 //! For example, suppose a C++ compilation unit has the definition:
 //!
@@ -22,9 +22,10 @@
 //!
 //! The Itanium C++ ABI specifies that the linker symbol for that function must
 //! be named `_ZN5space3fooEii`. This crate can parse that name into a Rust
-//! value representing its structure. Formatting the value with `format!` or
-//! `to_string` would yield the string `"space::foo(int, int)"`, which is more
-//! meaningful to the C++ developer.
+//! value representing its structure. Formatting the value with the `format!`
+//! macro or the `std::string::ToString::to_string` trait method yields the
+//! string `space::foo(int, int)`, which is more meaningful to the C++
+//! developer.
 
 #![deny(missing_docs)]
 #![deny(missing_debug_implementations)]
@@ -42,7 +43,7 @@ pub mod error;
 mod index_str;
 mod subs;
 
-use ast::{Demangle, Parse};
+use ast::{Demangle, Parse, ParseContext};
 use error::{Error, Result};
 use index_str::IndexStr;
 use std::fmt;
@@ -71,6 +72,7 @@ impl<T> Symbol<T>
     ///
     /// ```
     /// use cpp_demangle::Symbol;
+    /// use std::string::ToString;
     ///
     /// // First, something easy :)
     ///
@@ -79,7 +81,7 @@ impl<T> Symbol<T>
     /// let sym = Symbol::new(&mangled[..])
     ///     .expect("Could not parse mangled symbol!");
     ///
-    /// let demangled = format!("{}", sym);
+    /// let demangled = sym.to_string();
     /// assert_eq!(demangled, "space::foo(int, bool, char)");
     ///
     /// // Now let's try something a little more complicated!
@@ -90,7 +92,7 @@ impl<T> Symbol<T>
     /// let sym = Symbol::new(&mangled[..])
     ///     .expect("Could not parse mangled symbol!");
     ///
-    /// let demangled = format!("{}", sym);
+    /// let demangled = sym.to_string();
     /// assert_eq!(
     ///     demangled,
     ///     "JS_GetPropertyDescriptorById(JSContext*, JS::Handle<JSObject*>, JS::Handle<jsid>, JS::MutableHandle<JS::PropertyDescriptor>)"
@@ -100,8 +102,12 @@ impl<T> Symbol<T>
         let mut substitutions = subs::SubstitutionTable::new();
 
         let parsed = {
+            let ctx = ParseContext::default();
             let input = IndexStr::new(raw.as_ref());
-            let (parsed, tail) = try!(ast::MangledName::parse(&mut substitutions, input));
+
+            let (parsed, tail) = try!(ast::MangledName::parse(&ctx, &mut substitutions, input));
+            debug_assert!(ctx.recursion_level() == 0);
+
             if tail.is_empty() {
                 parsed
             } else {
@@ -140,6 +146,7 @@ impl<T> Symbol<T> {
     ///
     /// ```
     /// use cpp_demangle::BorrowedSymbol;
+    /// use std::string::ToString;
     ///
     /// let mangled = b"_ZN5space3fooEibc and some trailing junk";
     ///
@@ -148,14 +155,16 @@ impl<T> Symbol<T> {
     ///
     /// assert_eq!(tail, b" and some trailing junk");
     ///
-    /// let demangled = format!("{}", sym);
+    /// let demangled = sym.to_string();
     /// assert_eq!(demangled, "space::foo(int, bool, char)");
     /// ```
     pub fn with_tail(input: &[u8]) -> Result<(BorrowedSymbol, &[u8])> {
         let mut substitutions = subs::SubstitutionTable::new();
 
+        let ctx = ParseContext::default();
         let idx_str = IndexStr::new(input);
-        let (parsed, tail) = try!(ast::MangledName::parse(&mut substitutions, idx_str));
+        let (parsed, tail) = try!(ast::MangledName::parse(&ctx, &mut substitutions, idx_str));
+        debug_assert!(ctx.recursion_level() == 0);
 
         let symbol = Symbol {
             raw: input,
